	;program starts with jump to power-up initialization
		org 0000h
		ljmp ini_dsp
	;external interrupt 0 vector
		org 03h
		ajmp rtcint
	;timer 0 interrupt vector		
		org 0bh
		ajmp ctrint
 	;external interrupt 1 vector
		org 13h
		ajmp keyint
	;lcd initialization delay/instruction pairs
		org 80h
dat_ini:	dw 8038h		;>15ms delay, function set
		dw 2038h		;>4.1ms delay, function set
		dw 0238h		;>100us delay, function set
		dw 0138h		;>40us delay, function set
		dw 0106h		;>40us delay, entry mode set
		dw 010eh		;>40us delay, display on/off control
		dw 0101h		;>40us delay, display clear
		dw 1080h		;>1.4ms delay, DD ram address set
	;initial display data
dat_dsp:	db "2359:59:7:123199"	;lcd line 1 initialization
		db "GPC1 OS1   25.5V"	;lcd line 2 initialization
	;initial time and control bytes
dat_rtc:	db 00h
		db 59h
		db 59h
		db 23h
		db 07h
		db 31h
		db 12h
		db 99h
		db 80h
		db 00h
		db 00h
		db 00h
		db 00h
		db 00h
		db 00h
dat_con:	db 00h			;counter of t0 overflows (initial value for 74h)
		db 0ffh			;delay
		db 0ch			;keysound
		db 08h			;debounce
		db 0f0h			;lastkey
		db 84h			;cursorposition
		db 80h			;dacout
		db 0ffh			;last dac ch0 reading
		db 0ffh			;current dac ch0 reading

		;load ram from code
sub_wrm:	mov r0, #00h		;initialize offset
sub_wrm1:	mov a, r0		
		movc a, @a+dptr		;enter with code start address in dptr
		mov @r1, a		;enter with ram start address in r1
		inc r0
		inc r1
		djnz r2, sub_wrm1	;enter with number of bytes in r2
		ret

	;write the current character to the lcd module
sub_dc:		mov a, r1		;get current cursor position
		cjne a, #90h, sub_dc1	
sub_dc1:	jc sub_dc2		;jump if first line
		add a, #30h		;adjust LCD address if second line
sub_dc2:	mov r2, a		;lcd starting address to r2
		mov r3, #01h		;one character to write
	;write a string of characters to the lcd module
		;put the lcd cursor at the starting point
sub_dsp:	push 8eh		;save exram status in case calling program using exram
		orl 8eh, #03h		;set exram bit to enable external memory access
		mov r0, #00h		;r0 could be any any number since lcd does not latch address
		acall sub_dsp2		;wait until lcd not busy
		clr p2.3		;select write to lcd
		mov a, r2		;enter with lcd starting address in r2
		movx @r0, a		;write starting address
		;write the data to the lcd
sub_dsp1:	acall sub_dsp2		;wait until lcd not busy
		setb p2.2		;select data register
		clr p2.3		;select write to lcd
		mov a, @r1		;enter with address of first character in r1
		movx @r0, a		;write a data byte to lcd
		inc r1			;address next byte
		djnz r3, sub_dsp1	;dec number of characters in r3 and loop til done
 	        pop 8eh			;restore the exram status to avoid double push
		;restore the cursor position
sub_wc:		push 8eh		;save the exram status when this sub used alone
		acall sub_dsp2		;wait until lcd not busy
		clr p2.3		;select write to lcd
		mov a, 79h		;get cursor position
		cjne a, #90h, sub_wc1	;convert cursor position to LCD notation
sub_wc1:	jc sub_wc2 
		add a, #30h
sub_wc2:	movx @r0, a		;write the cursor position
		pop 8eh			;restore exram status
		ret
		;subroutine waits until lcd not busy
sub_dsp2:	clr p2.2		;select instruction register
		setb p2.3		;select read from lcd
sub_dsp4:	movx a, @r0		;get status
		jb 0e7h, sub_dsp4	;wait until not busy
		ret
	;i2c transfer 
sub_i2c:	mov 0d8h, #00h
		mov 0d8h, #40h		;initialize S1CON
		setb 0ddh		;transmit start condition
		mov a, #08h		;expected response
		acall sub_i2c6		;wait for response
		clr 0ddh		;clear STA
		mov a, r1		;get device address
		mov 0dah, a		;address to S1DAT
		clr 0dbh		;clear SI to send the device address
		jnb 0e0h, sub_i2c3	;test r/w bit and jump if clear to write data		
		;read data		
		mov a, #40h		;expected response
		acall sub_i2c6		;wait for response
		setb 0dah		;set AA
sub_i2c1:	djnz r3, sub_i2c2	;jump if not last byte
		clr 0dah		;clear AA for last byte
		clr 0dbh		;clear SI to receive the last byte
		mov a, #58h		;expected response
		acall sub_i2c6		;wait for response
		mov @r0, 0dah		;save the last byte
		mov 0d8h, #50h		;stop and clear AA
		ret			;all bytes received
sub_i2c2:	clr 0dbh		;clear SI to receive a byte
		mov a, #50h		;expected response
		acall sub_i2c6		;wait for response
		mov @r0, 0dah		;save the byte
		inc r0			;address next byte
		sjmp sub_i2c1		;loop until done
		;write data
sub_i2c3:	mov a, #18h		;expected response
		acall sub_i2c6		;wait for response
		mov 0dah, r2		;load the control byte
sub_i2c4:	clr 0dbh		;clear SI to transmit the byte
		mov a, #28h		;expected response
		acall sub_i2c6		;wait for response
		cjne r3, #00h, sub_i2c5	;jump if more bytes to transmit
		mov 0d8h, #50h		;stop and release buss
		ret
sub_i2c5:	mov 0dah, @r0		;load a data byte
		inc r0			;address next byte
		dec r3			;decrement byte counter
		sjmp sub_i2c4		;loop until done
		;wait for response
sub_i2c6:	mov r4, #0ffh		;initialize timeout
sub_i2c7:	jnb 0dbh, sub_i2c8	;wait for SI
		cjne a, 0d9h, sub_i2c9	;jump if not expected response
		ret			;return if expected response
sub_i2c8:	djnz r4, sub_i2c7	;loop if not timed out
sub_i2c9:	pop acc			;pop the subroutine return address
		pop acc
		ret			;abort the i2c transfer



	;start timer 0 and set up interrupt counter
sub_ctr:	orl tmod, #00h		;put timer 0
		anl tmod, #0f0h		;in mode 0
		mov tl0, #00h		;clear high
		mov th0, #00h		;and low bytes
		mov 74h, #00h		;clear counter
		setb 8ch		;start timer 0
		anl tcon, #0d7h		;clear t0, int1 flags
		setb 0a9h		;enable t0 int
		clr 0aah		;disable int1 int
		ret
		

sub_ky:		mov r0, #79h		;address of cursor position
		mov a, @r0
		mov r1, a
		mov a, 78h		;get the last key
		cjne a, #0e0h, sub_ky2	;jump if not cursor left key
		dec @r0			;decrement the cursor position
		cjne @r0, #7Fh, sub_ky1
		mov @r0, #9ah		;if beginning of first line, move to end of second line  
sub_ky1:	ajmp sub_wc		;write cursor position to LCD and exit 
sub_ky2:	cjne a, #70h, sub_ky4	;jump if not cursor right key
		inc @r0			;increment the cursor position
		cjne @r0, #9bh, sub_ky3
		mov @r0, #80h		;if end of second line, move to beginning of first line
sub_ky3:	ajmp sub_wc		;write cursor position to LCD and exit
sub_ky4:	cjne a, #0d0h, sub_ky5	;jump if not increment key
		mov a, #01h
		sjmp sub_ky6
sub_ky5:	cjne a, #0b0h, sub_ky7	;jump if not decrement key	
		mov a, #0ffh
sub_ky6:	cjne r1, #84h, sub_ky8	;jump if cursor not home
		ajmp sub_ky21		;jump to mode change
sub_ky7:	ret			;return if no multiple key detection needed
sub_ky8:	cjne r1, #87h, sub_ky13 ;jump if cursor not at backlight indicator
		cjne @r1, #3ah, sub_ky11;jump if indicator not a :
		mov @r1, #2dh		;change indicator to a -
		clr p2.0		;turn off backlight
		sjmp sub_ky12
sub_ky11:	mov @r1, #3ah		;change indicator to a :	
		setb p2.0		;turn on backlight
sub_ky12:	ajmp sub_dc		;write the new indicator and exit	
sub_ky13:	mov r0, #84h
		cjne r1, #89h, sub_ky15 ;jump if cursor not at alarm indicator
		cjne @r0, #2ah, sub_ky17;jump to ret if mode not * (alarm set mode)
		clr 0a8h		;disable int0 (stops rtc interrupts)
		cjne @r1, #2ah, sub_ky14;jump if indicator not * (alarm on)
		mov @r1, #3ah		;change alarm indicator to : (alarm off)
		sjmp sub_ky12
sub_ky14:	mov @r1, #2ah		;change alarm indicator to * (alarm on)
		sjmp sub_ky12		;write the new indicator and exit 
sub_ky15:	add a, @r1		;modify the data up or down by 1
		cjne r1, #90h, sub_ky16
sub_ky16:	jnc sub_ky20		;jump if not top line
		cjne @r0, #3ah, sub_ky18;jump if mode not : 				
sub_ky17:	ret		
sub_ky18:	clr 0a8h		;disable int0 (stops rtc interrupts)
		cjne a, #3ah, sub_ky19
		mov a, #30h		;increment from 9 to 0
		sjmp sub_ky20
sub_ky19:	cjne a, #2fh, sub_ky20	;decrement from 0 to 9
		mov a, #39h
sub_ky20:	mov @r1, a		;save the new data
		ajmp sub_dc		;write the new data to LCD and exit
sub_ky21:	cjne @r1, #3ah, sub_ky22;jump if mode not :
		inc a
		jnz sub_ky23		;jump if increment
		mov @r1, #2dh		;set mode to - (stop the clock)
		ajmp sub_dc		;write the mode to LCD and exit
sub_ky22:	ajmp sub_ky42
sub_ky23:	mov @r1, #2ah		;set mode to # (alarm set mode)
		mov r0, #68h		;address the calibration data
		mov r1, #8fh		;first calibration octal digit
		mov r2, #0fh		;mask data
		mov r4, #30h		;to convert to ascii
		mov a, @r0
		anl a, #07h
		orl a, r4
		mov @r1, a		;unpack the low cal octal digit
		dec r1
		mov a, @r0
		rr a
		rr a
		rr a
		anl a, #07h		
		orl a, r4
		mov @r1, a		;unpack the high cal octal digit
		mov r0, #6bh		;date
		dec r1			;address units of days
		mov a, @r0
		anl a, r2
		orl a, r4
		mov @r1, a		;unpack the units of days
		mov a, @r0
		swap a
		dec r1			;address tens of days
		anl a, #03h
		orl a, r4
		mov @r1, a		;unpack the tens of days
		mov a, @r0
		anl a, #0c0h
		swap a
		mov b, a		;save repeat bits 4 and 5 in b
		dec r1			;address units of months
		dec r0			;address months
		mov a, @r0
sub_ky24:	anl a, r2
		orl a, r4
		mov @r1, a		;unpack the units of months
		dec r1
		mov a, @r0
		swap a
		anl a, #01h
		orl a, r4
		mov @r1, a		;unpack the tens of months
		dec r1   		;skip the alarm status
		mov r0, #6eh		;address seconds
		mov r1, #86h		;address units of seconds
		mov a, @r0
		anl a, r2
		orl a, r4
		mov @r1, a		;unpack the units of seconds
		dec r1
		mov a, @r0
		jnb 0e7h, sub_ky25
		setb 0f6h		;save repeat bit 1 in b
sub_ky25:	swap a
		anl a, #07h
		orl a, r4
		mov @r1, a		;unpack the tens of seconds
		dec r1
		dec r1			;skip the mode character
		dec r0
		mov a, @r0
		anl a, r2
		orl a, r4
		mov @r1, a		;unpack the units of minutes
		dec r1
		mov a, @r0
		jnb 0e7h, sub_ky26
		setb 0f5h		;save repeat bit 2 in b
sub_ky26:	swap a
		anl a, #07h
		orl a, r4
		mov @r1, a		;unpack the tens of minutes
		dec r1
		dec r0
		mov a, @r0
		anl a, r2
		orl a, r4
		mov @r1, a		;unpack the units of hours
		dec r1
		mov a, @r0
		jnb 0e7h, sub_ky27
		setb 0f4h		;save repeat bit 3 in b
sub_ky27:	swap a
		anl a, #03h
		orl a, r4
		mov @r1, a		;unpack the tens of hours
		mov r1, #88h		;address the repeat character
		mov a, b		;get the repeat bits
		mov @r1, #30h
		cjne a, #00h, sub_ky28  ;unpack 0 to repeat character for once per year
		sjmp sub_ky34
sub_ky28:	inc @r1
		cjne a, #04h, sub_ky29	;unpack 1 for once per month
		sjmp sub_ky34
sub_ky29:	inc @r1
		cjne a, #0ch, sub_ky30	;unpack 2 for once per day
		sjmp sub_ky34
sub_ky30:	inc @r1
		cjne a, #1ch, sub_ky31	;unpack 3 for once per hour
		sjmp sub_ky34
sub_ky31:	inc @r1
		cjne a, #3ch, sub_ky32	;unpack 4 for once per min
		sjmp sub_ky34
sub_ky32:	inc @r1
		cjne a, #7ch, sub_ky33	;unpack 5 for once per min
		sjmp sub_ky34
sub_ky33:	inc @r1			;inc to 6 if error
sub_ky34:	mov r1, #80h		;LCD starting address
		mov r2, #80h		;address of first character to write
		mov r3, #10h		;write 16 characters
		ajmp sub_dsp		;write the top line to the LCD and exit

sub_ky42:	cjne @r1, #2ah, sub_ky43;jump if mode not *
		inc a
		jz sub_ky44		;jump if decrement
		ret
sub_ky43: 	ajmp sub_ky60
sub_ky44:	mov @r1, #3ah		;decrement to : (clock mode), will update in one second
		jnb 0a8h, sub_ky45	;jump if int0 is disabled
		ret
sub_ky45:	mov r1, #88h		;address the repeat character
		mov a, @r1		;get the repeat character
		mov b, #7ch		;all repeat bits set
		cjne a, #35h, sub_ky46	;separate the repeat bits in b
		sjmp sub_ky51
sub_ky46:	clr 0f6h
		cjne a, #34h, sub_ky47
		sjmp sub_ky51
sub_ky47:	clr 0f5h
		cjne a, #33h, sub_ky48
		sjmp sub_ky51
sub_ky48:	clr 0f4h
		cjne a, #32h, sub_ky49
		sjmp sub_ky51
sub_ky49:	clr 0f3h
		cjne a, #31h, sub_ky50
		sjmp sub_ky51
sub_ky50:	clr 0f2h	
sub_ky51:	mov r0, #68h		;address the calibration byte
		mov r1, #8fh		;address the low octal digit of calibration
		mov r2, #0fh		;4 bit mask
		mov r3, #07h		;3 bit mask
		mov a, @r1
		anl a, r3
		mov r4, a		;save the low octal digit
		dec r1
		mov a, @r1
 	        anl a, r3
		rl a
		rl a
		rl a
		orl a, r4		;combine low and high octal digits
		setb 0e7h		;set OUT high
		mov @r0, a		;pack the calibration byte
		dec r1
		inc r0
		mov @r0, #00h		;clear the watchdog counter
		inc r0
		inc r0
		mov a, @r1		;get units of days
		anl a, r2
		mov r4, a
		dec r1
		mov a, @r1		;get tens of days
		anl a, #03h
		swap a
		orl a, r4
		mov r4, a
		mov a, b		;get repeat bits
		anl a, #0ch
		swap a
		orl a, r4
		mov @r0, a		;pack the days
		dec r0
		dec r1
		mov a, @r1		;get units of months
		anl a, r2
		mov r4, a
		dec r1
		mov a, @r1		;get tens of months
		anl a, #01h
		swap a
		orl a, r4
		dec r1
		clr 0e7h		;clear AFE
		cjne @r1, #2ah, sub_ky52;jump if alarm not set
		setb 0e7h		;set AFE (alarm flag enable)
sub_ky52:	setb 0e5h		;set ABE (lets alarm work in battery back-up mode)
		mov @r0, a		;pack the months
		dec r1
		dec r1
		mov r0, #73h		;address the sqw bits
		mov a, @r0
		setb 0e4h
		cjne @r1, #3ah, sub_ky53;jump if backlight is off
		clr 0e4h
sub_ky53:	mov @r0, a		;pack the backlight status bit (low is backlight on)
		mov r0, #6eh		;address seconds byte
		mov r1, #86h		;address units of seconds
		mov a, @r1
		anl a, r2
		mov r4, a
		dec r1			;address tens of seconds
		mov a, @r1		
		anl a, r3
		swap a
		orl a, r4
		jnb 0f6h, sub_ky54
		setb 0e7h
sub_ky54:	mov @r0, a		;pack the seconds
		dec r1
		dec r1			;address units of minutes
		dec r0			;address minutes byte
		mov a, @r1
		anl a, r2
		mov r4, a
		dec r1			;address tens of minutes
		mov a, @r1
		anl a, r3
		swap a
		orl a, r4
		jnb 0f5h, sub_ky55
 	        setb 0e7h
sub_ky55:	mov @r0, a		;pack the minutes
		dec r1			;address units of hours
		dec r0 			;address hours byte
		mov a, @r1
		anl a, r2
		mov r4, a
		dec r1			;address tens of hours
		mov a, @r1
		anl a, #03h
		swap  a
		orl a, r4
		jnb 0f4h, sub_ky56
		setb 0e7h
sub_ky56:	mov @r0, a		;pack the hours
		mov r0, #68h
		mov r1, #0d0h
		mov r2, #08h
		mov r3, #07h
		acall sub_i2c		;write alarm adata and clear watchdog timer
		mov r0, #73h
		mov r1, #0d0h
		mov r2, #13h
		mov r3, #01h
		acall sub_i2c		;write sqw bits (bit 4 or RS0 is backlight status)
		mov r1, #0d0h
		mov r2, #0fh
		mov r3, #00h
		acall sub_i2c		;address the flags
		mov r0, #6fh
		mov r1, #0d1h
		mov r3, #01h
		acall sub_i2c		;read the flags to clear
		mov 69h, #06h		;set the watchdog ctr for 1 second
		mov r0, #69h
		mov r1, #0d0h
		mov r2, #09h
		mov r3, #01h
		acall sub_i2c		;write to watchdog
		setb 0a8h		;enable int0
		ret

sub_ky60:	inc a
		jnz sub_ky61
		ret
sub_ky61:	mov @r1, #3ah		;set mode to :
		jnb 0a8h, sub_ky62	;jump if int0 is disabled
		ret	
sub_ky62:	mov r0, #67h		;address the years byte
		mov r1, #8fh		;address units of years
		mov r2, #0fh		;4 bit mask
		mov r3, #07h		;3 bit mask
		mov a, @r1
		anl a, r2
		mov r4, a
		dec r1
		mov a, @r1
 	        anl a, r2
		swap a
		orl a, r4
		mov @r0, a		;pack the years
		dec r1
		dec r0
		dec r0
		mov a, @r1		;get units of days
		anl a, r2
		mov r4, a
		dec r1
		mov a, @r1		;get tens of days
		anl a, #03h
		swap a
		orl a, r4
		mov @r0, a		;pack the days
		inc r0
		dec r1
		mov a, @r1		;get units of months
		anl a, r2
		mov r4, a
		dec r1
		mov a, @r1		;get tens of months
		anl a, #01h
		swap a
		orl a, r4
		mov @r0, a		;pack the months
		dec r1
		dec r1
		dec r0
		dec r0
		mov a, @r1
		anl a, r3
		mov @r0, a		;pack the day byte
		mov r0, #61h		;address seconds byte
		mov r1, #86h		;address units of seconds
		mov a, @r1
		anl a, r2
		mov r4, a
		dec r1			;address tens of seconds
		mov a, @r1		
		anl a, r2		;the 4 bit mask allows the stop bit to be set for 8x or 9x seconds
		swap a
		orl a, r4
		mov @r0, a		;pack the seconds
		dec r1
		dec r1			;address units of minutes
		inc r0			;address minutes byte
		mov a, @r1
		anl a, r2
		mov r4, a
		dec r1			;address tens of minutes
		mov a, @r1
		anl a, r3
		swap a
		orl a, r4
		mov @r0, a		;pack the minutes
		dec r1			;address units of hours
		inc r0 			;address hours byte
		mov a, @r1
		anl a, r2
		mov r4, a
		dec r1			;address tens of hours
		mov a, @r1
		anl a, #03h
		swap  a
		orl a, r4
		mov @r0, a		;pack the hours
		mov 69h, #00h		;clear the watchdog
		mov r0, #61h
		mov r1, #0d0h
		mov r2, #01h
		mov r3, #09h
		acall sub_i2c		;write time and clear watchdog timer
		mov r1, #0d0h
		mov r2, #0fh
		mov r3, #00h
		acall sub_i2c		;address the flags
		mov r0, #6fh
		mov r1, #0d1h
		mov r3, #01h
		acall sub_i2c		;read the flags to clear
		mov 69h, #06h		;set the watchdog ctr for 1 second
		mov r0, #69h
		mov r1, #0d0h
		mov r2, #09h
		mov r3, #01h
		acall sub_i2c		;write the watchdog
		setb 0a8h		;enable int0
		ret


	;power-up initialization starts here
ini_dsp:	mov sp, #0c0h		;set stack pointer
		clr  p2.2		;select instruction register
		clr  p2.3		;select write to lcd
		mov dptr, #dat_ini	;table of delays and intructions
		mov r3, #00h		;initialize table pointer
		mov r2, #08h		;number of delay/instruction pairs
ini_dsp1:	mov a, r3
		mov r0, #0ffh
		movc a, @a+dptr		;get delay
		mov r1, a
ini_dsp2:	djnz r0, ini_dsp2
		djnz r1, ini_dsp2	;wait delay time
		inc r3			;advance pointer to instruction
		mov a, r3
		movc a, @a+dptr		;get instruction
		movx @r1, a		;write instruction to lcd
		inc r3			;advance pointer to next delay
		djnz r2, ini_dsp1	;repeat
		;initialize the display data
ini_lcd:  	mov dptr, #dat_dsp	;start of display initialization data
		mov r2, #20h		;number of bytes to transfer
		mov r1, #80h		;ram start address
		acall sub_wrm		;write ram
		;initialize control characters
		mov dptr, #dat_con	;start of control initialization data
		mov r2, #09h		;number of bytes to transfer
		mov r1, #74h		;ram start address
		acall sub_wrm		;write ram 
		;initialize power down counter
		mov 1fh, #10		;10 second power down delay to r7 of bank 3
		;initialize the special function registers
ini_sfr:	mov 0a8h, #05h		;disable interrupts, unmask int0 and int1
		mov ip, #0dch		;set interrupt priorities, 11=3=highest, 00=0=lowest 
		mov 0b7h, #0d9h		;t2=11, pca=11, i2c=00, serial=11, t1=11, int0=10, t0=00, int0=01
		;prepare an initial rtc alarm interrupt	
		mov tcon, #07h		;set int0 flag to insure an alarm int on power-up
		setb 0afh		;enable interrupts
		;jump to the user program at 800h	
		ljmp userprog

	;rtc alarm has caused an interrupt on int0
rtcint:		push acc
		push b
		push psw		
		orl psw, #18h		;select register bank 3		
		inc 0a2h		;select dptr 2
		;address first byte
		mov r1, #0d0h		;rtc write address
		mov r2, #00h		;rtc word address
		mov r3, #00h		;number of data bytes to write
		acall sub_i2c		;address word 00
		;get all rtc data
		mov r0, #60h		;rtc data destination
		mov r1, #0d1h		;rtc read address
		mov r3, #14h		;number of data bytes to read
		acall sub_i2c		;read 20 characters from rtc
		;test the ST bit
		mov a, 61h		;get seconds byte
		jnb 0e7h, rtcint1	;skip rtc initialization if ST not set
		;initialize rtc buffer
		mov dptr, #dat_rtc	;start of time initialization data
		mov r2, #0fh		;number of bytes to transfer
		mov r1, #60h		;start of time buffer in ram
		acall sub_wrm		;write ram
		;write initialization data to rtc
		mov r0, #61h
		mov r1, #0d0h		;rtc write address
		mov r2, #01h		;rtc word address
		mov r3, #0eh		;number of data bytes to write
		acall sub_i2c		;write buffer to rtc
		;clear watchdog timer
rtcint1:	mov 69h, #00h		;clear watchdog timer
		;write to rtc
		mov r0, #069h		;rtc data source
		mov r1, #0d0h		;rtc write address
		mov r2, #09h		;rtc word address
	 	mov r3, #01h		;number of bytes to write		
		acall sub_i2c		;write clear watchdog to rtc
		;reset watchdog timer
		mov 69h, #06h		;reset watchdog timer for 1 second
		mov r0, #69h		
		mov r1, #0d0h
		mov r2, #09h
		mov r3, #01h	
		acall sub_i2c
		;test HT bit
		mov a, 6ch
		jnb 0e6h, rtcint4	;jump if halt not set
		anl 6ch, #0bfh		;clear halt in buffer
		;write to rtc to clear halt
		mov r0, #6ch		
		mov r1, #0d0h		;rtc write address
		mov r2, #0ch		;rtc word address
	 	mov r3, #01h		;number of bytes to write		
		acall sub_i2c		;write clear halt to rtc
		orl 6ch, #40h		;restore halt bit in buffer for monitoring
		;get backlight status
		mov a, 73h		;get the sqw byte (RS0 or bit 4 is the backlight status bit)
		jnb 0e4h, rtcint2	;jump if backlight status bit clear (backlight on)
		mov r0, #87h		;address backlight character
		mov @r0, #2dh		;write - to backlight character
		clr p2.0		;turn off backlight
		;test AFE bit
rtcint2:	mov a, 6ah		;get the alarm month byte
		jnb 0e7h, rtcint3	;jump if AFE clear
		mov r0, #89h
		mov @r0, #2ah		;write * to alarm status 
		sjmp rtcint6		;jump to alarm sound
rtcint3:	mov r0, #84h		;address display mode
		mov @r0, #2dh		;write a - to display mode
		ajmp rtcint8
		;test alarm flag
rtcint4:	mov a, 6fh		;get rtc flags
		jb 0e6h, rtcint6	;jump if alarm flag
		mov a, 6ah		;get the alarm month byte
		jnb 0e7h, rtcint7	;jump if AFE clear
		djnz 1fh, rtcint7	;decrement power down counter and jump if not 0
		clr p2.1		;power down
rtcint5:	sjmp rtcint5		;wait for power down		
		;alarm sound
rtcint6:	jb 8ch, rtcint7		;jump if timer 0 in use 
		orl 0a2h, #08h		;set soundalarm flag 
		mov 075h, #80h		;set delay for alarm duration of .5 second
		acall sub_ctr		;start timer 0
		clr p3.3		;turn on sound
		;test for mode
rtcint7:	mov r0, #84h
		cjne @r0, #3ah, rtcint9	;if mode not : don't update time	
		;format the time data for display
rtcint8:	mov r0, #61h
		mov r1, #86h		
		mov r2, #0fh
		mov r3, #30h
		mov a, @r0
		anl a, r2
		orl a, r3
		mov @r1, a		;units of seconds
		mov a, @r0
		swap a
		anl a, r2
		orl a, r3
		dec r1
		mov @r1, a		;tens of seconds
		inc r0
		mov a, @r0
		anl a, r2
		orl a, r3
		dec r1
		dec r1
		mov @r1, a		;units of minutes
		mov a, @r0
		swap a
		anl a, r2
		orl a, r3
		dec r1
		mov @r1, a		;tens of minutes
		inc r0
		mov a, @r0
		anl a, r2
		orl a, r3
		dec r1
		mov @r1, a		;units of hours
		mov a, @r0
		swap a
		anl a, r2
		orl a, r3
		dec r1
		mov @r1, a		;tens of hours
		inc r0
		mov a, @r0
		anl a, r2
		orl a, r3
		mov r1, #88h
		mov @r1, a		;day of week
		inc r0
		mov a, @r0
		anl a, r2
		orl a, r3
		mov r1, #8dh
		mov @r1, a		;units of date
		mov a, @r0
		swap a
		anl a, r2
		orl a, r3
		dec r1
		mov @r1, a		;tens of date
		inc r0
		mov a, @r0
		anl a, r2
		orl a, r3
		dec r1
		mov @r1, a		;units of month
		mov a, @r0
		swap a
		anl a, r2
		orl a, r3
		dec r1
		mov @r1, a		;tens of month
		inc r0
		mov a, @r0
		anl a, r2
		orl a, r3
		mov r1, #8fh
		mov @r1, a		;units of year
		mov a, @r0
		swap a
		anl a, r2
		orl a, r3
		dec r1
		mov @r1, a		;tens of year
		;write time to lcd
		mov r1, #80h		;location of data
		mov r2, #80h		;LCD start address
		mov r3, #10h		;number of characters to write
		acall sub_dsp		; write data to lcd
		;write adc control and dac bytes
rtcint9:	mov r0, #7ah		;dac byte
		mov r1, #9ah		;adc write address
		mov r2, #44h		;adc control byte 
		mov r3, #01h		;number of data bytes to write
		acall sub_i2c		;send control and dac bytes to adc
		;get adc data
		mov r0, #7bh		;adc data destination
		mov r1, #9bh		;adc read address
		mov r3, #05h		;number of data bytes to read
		acall sub_i2c		;read 5 characters from adc
		;check battery voltage for auto backlignt off and auto power down
		mov a, 7ch		;get adc ch0 data
		cjne a, #55, rtcint10
rtcint10:	jnc rtcint13		;jump if input greater than 5.5V
		mov r1, #87h		;address BL indicator
		cjne @r1, #2dh, rtcint12;jump if BL indicator not a - (BL on)
		anl 6ah, #7fh		;turn off AFE
		mov r0, #06ah		;rtc data source
		mov r1, #0d0h		;rtc write address
		mov r2, #0ah		;rtc word address
	 	mov r3, #01h		;number of bytes to write		
		acall sub_i2c		;write to RTC to turn off alarm		
		clr p2.1		;power down
rtcint11:	sjmp rtcint11		;wait for power down
rtcint12:	mov @r1, #2dh		;change BL indicator to -
		clr p2.0		;turn off BL		
		;convert channel 0 adc data to BCD
rtcint13:	mov r0, #9bh		;input volts location in display buffer
		mov r2, #30h
		mov a, 7ch		;get adc ch0 data
		mov b, #64h
		div ab			;divide by 100
		orl a, r2
		mov @r0, a		;write tens of volts
		mov a, b
		mov b, #0ah
		div ab			;divide by 10
		orl a, r2
		inc r0
		mov @r0, a		;write units of volts
		mov a, b
		orl a, r2
		inc r0			;step over decimal point
		inc r0
		mov @r0, a		;write tenths of volts
		;write line 2 with input voltage to lcd
		mov r1, #90h		;location of data
		mov r2, #0c0h		;LCD start address
		mov r3, #10h		;number of characters to write
		acall sub_dsp		; write to LCD
		;restore registers
 		inc 0a2h		;select dptr 1
		pop psw
		pop b
		pop acc
		clr 89h			;clear the int0 flag
		reti

	;soundalarm or keypress has enabled t0 interrupts
ctrint:		push acc
		push b
		push psw
		inc 0a2h		;select dptr 2
		orl psw, #18h		;select register bank 3		
		mov r0, #74h		;address of counter in r0
		inc @r0			;increment the counter
		mov a, 0a2h		;get aux register
		jnb 0e3h, ctrint1	;jump if alarmsound not set
		;interrupt is due to alarmsound set
		mov a, p2		;get switch status
		add a, #10h		;will cause a carry if no key is pressed
		jnc ctrint8		;stop alarmsound if a key is pressed
		mov a, 75h		;get alarm duration from delay
		cjne a, 74h, ctrint9	;done if counter not equal to alarm duration
		ajmp ctrint8		;stop alarmsound
		;interrupt is due to a keypress
ctrint1:	mov a, @r0		;get counter
		cjne a, 077h, ctrint3	;jump if counter not equal debounce
		mov a, p2		;get switch status
		anl a, #0f0h		;clear bottom nibble
		cjne a, 78h, ctrint2	;compare new key with lastkey
		ajmp ctrint8		;if same, stop count
ctrint2:	mov 78h, a		;if not same, save new lastkey
		ajmp ctrint7		;process the key
ctrint3:	jc ctrint9		;done if counter less than debounce
		mov a, p2		;get swtich status
		anl a, #0f0h		;clear bottom nibble
		cjne a, 78h, ctrint8	;stop count if not same as lastkey
		mov a, @r0		;get counter
		cjne a, 76h, ctrint4	;jump if count not equal to keysound 
		setb p3.3		;turn sound off
ctrint4:	cjne a, 75h, ctrint9	;end if count not equal to delay
		mov @r0, 77h		;debounce to counter
		mov 75h, #20h		;load repeat delay
	;process the key
ctrint7:	clr p3.3		;turn sound on
		acall sub_ky
		ajmp ctrint9		;done
	;stop the count
ctrint8:	setb p3.3		;turn sound off
		anl 0a2h, #0f7h		;clear alarmsound flag
		clr 8ch			;stop timer 0
		clr 0a9h		;disable t0 int
		setb 0aah		;enable int1 int
	;done
ctrint9:	inc 0a2h		;select dptr 1
		pop psw			;restore registers
		pop b
		pop acc		
		reti

					
	;a keypress has caused a int1 interrupt
keyint:		mov 78h, #0f0h		;clear lastkey
		mov 75h, #80h		;set initial delay
		acall sub_ctr		;start counter
		reti


		


		

;User program starts at 800h. Resources being used by GPC1TST2 are external interrupts ex0 and ex1, t0,
;dptr 2, ram locations 60h through 9fh, instruction code locations through 7ffh, the general purpose
;flag at bit 3 of SFR 0a2h, and register bank 3.
;To run GPC1TST1 in the background, leave interrupts int0, int1, and t0 enabled.
;Some interrupts can take up to 20ms so the serial port is given highest priority.
;RAM 80h-8fh contains an image of LCD line 1 addresses 80h-8fh
;RAM 90h-9fh contains an image of LCD line 2 addresses c0h-cfh
;RAM 60h-73h contains an image of RTC data locations 00h-14h
;RAM 74h-79h contains program variables as follows:
	;counter=74h is the t0 interrupt counter
	;delay=75h is the duration of the key repeat rate or alarm sound
	;keysound=76h is the duration of the key press sound
	;debounce=77h is the duration of the key debounce wait
	;lastkey=78h is the last key pressed 
	;cursor=79h is the current lcd cursor position
;RAM 7ah contains the data sent to the DAC each second
;RAM 7bh contains the previous ADC CH0 data
;RAM 7ch-7fh contains the current ADC CH0-CH3 data, CH0 is the power input voltage displayed on the LCD

		org 800h
userprog:	ajmp userprog
		end
		


		
